/*
 *
 * Copyright (c) 2000 Scott Oaks and Henry Wong. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.rmi.*;
import net.jini.core.lease.*;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;
import net.jini.space.*;
import com.sun.jini.lease.*;

public class Converter {

    public static void main(String[] args) throws Exception {
        System.setSecurityManager(new RMISecurityManager());

        // Find the transaction manager
        ServiceFinder sf = new ServiceFinder(TransactionManager.class);
        TransactionManager txm = (TransactionManager)(sf.getObject());
        LeaseRenewalManager lrm = new LeaseRenewalManager();

        // Find the JavaSpaces
        ServiceFinder sfjs = new ServiceFinder(JavaSpace.class);
        JavaSpace js = (JavaSpace) sfjs.getObject();

        Conversion request = new Conversion(false);
        Conversion result;

        while (true) {
            Transaction.Created txn = null;
            try {
                // Take the request in a thread so that if the service
                // exits before it writes the answer the request is not
                // lost
                txn = TransactionFactory.create(txm, Lease.FOREVER);
                lrm.renewUntil(txn.lease, Lease.FOREVER, null);
                result = (Conversion) js.take(request,
                                        txn.transaction, Long.MAX_VALUE);

                // Calculate the answer
                try {
                    result.result = String.valueOf(result.value);
                } catch (Exception e) {
                    // If we can't calculate the answer, we have to
                    // abort the transaction, but we can go get the
                    // next request
                    txn.transaction.abort();
                    lrm.remove(txn.lease);
                    continue;
                }

                // Write the answer
                result.done = new Boolean(true);
                js.write(result, txn.transaction, 5 * 60 * 1000);
                txn.transaction.commit();
                lrm.remove(txn.lease);
            } catch (Exception e) {
                // These exceptions mean that we couldn't talk to the
                // space or transaction manager, which we can't
                // recover from.
                System.out.println("Unrecoverable error" + e);
                System.exit(-1);
            }
        }
    }
}
